class NodeBaseline < ActiveRecord::Base
  unloadable

  belongs_to :node
  belongs_to :reference_node, :class_name => "Node"
  belongs_to :baseline

  has_many :differences, :class_name => "BaselineDifference", :dependent => :delete_all
  def can_change_all?
    BaselineDifference.can_change_all? differences
  end

  validates_presence_of :node_id, :date, :reference_node_id
  validates_uniqueness_of :node_id, :scope => [:date, :reference_node_id]

  named_scope :self_comparisons, :conditions => "`node_id` = `reference_node_id`"
  named_scope :newest_first, :order => 'date DESC'
  named_scope :diffed, :conditions => { :diffed => true }

  def previous
    NodeBaseline.
      first(:conditions => ['node_id = ? AND reference_node_id = ? AND date < ?',
                            node_id, reference_node_id, date],
            :order => 'date DESC')
  end

  named_scope :for_date, lambda { |date|
    { :joins => :differences, :conditions => { :date => date } }
  }

  named_scope :with_diffs_by_status, lambda { |accepted|
    {
      :joins => :differences,
      :conditions => { :baseline_differences => { :accepted => accepted }},
    }
  }

  def self.unreviewed_node_count_for_date(date)
    self_comparisons.for_date(date).count('node_id', :distinct => true, :conditions => { :baseline_differences => { :accepted => nil } })
  end

  def unreviewed_resource_count
    differences.count(:conditions => { :baseline_differences => { :accepted => nil } })
  end

  def accepted_resource_count
    differences.count(:conditions => { :baseline_differences => { :accepted => true } })
  end

  def rejected_resource_count
    differences.count(:conditions => { :baseline_differences => { :accepted => false } })
  end

  def resource_counts_by_status
    counts = differences.count(:group => :accepted).to_hash
    {
      'accepted'   => counts[true].to_i,
      'rejected'   => counts[false].to_i,
      'unreviewed' => counts[nil].to_i
    }
  end

  def carry_over_to_tomorrow
    NodeBaseline.create!(:date => self.date.tomorrow, :baseline_id => self.baseline_id, :node_id => self.node_id, :reference_node_id => self.reference_node_id, :diffed => false)
  end

  def clone_and_apply_changes(changes=[])
    new_node_baseline = nil
    ActiveRecord::Base.transaction do
      new_baseline = self.baseline.deep_clone
      new_node_baseline = NodeBaseline.create!(:date => self.date.tomorrow, :baseline_id => new_baseline.id, :node_id => self.node_id, :reference_node_id => self.reference_node_id, :diffed => false)

      changes.each do |change|
        change.apply_decision_to(new_baseline)
      end
    end

    new_node_baseline
  end

  def create_for_next_day
    changes = BaselineDifference.pending_changes_for_node_on_date(node_id, date)

    if changes.any? {|c| c.accepted }
      clone_and_apply_changes changes
    else
      carry_over_to_tomorrow
    end
  end

  def self.create_from_report!(report, time)
    date = time.to_date
    baseline = Baseline.create_from_report!(report)
    NodeBaseline.create!(:date => date, :baseline_id => baseline.id, :node_id => report.node_id, :reference_node_id => report.node_id, :diffed => false)
  end

  def self.delete_older_than(date)
    self.destroy_all(['date < ?', date])
    baselines = Baseline.all(:joins => "LEFT OUTER JOIN node_baselines ON node_baselines.baseline_id = baselines.id",
                             :conditions => {:node_baselines => {:baseline_id => nil}})
    baselines.each do |baseline|
      baseline.destroy
      yield if block_given?
    end
  end

  def self.resource_counts_for_group_and_date(group, date, accepted)
    resource_counts_for_node_ids_and_date(group.nodes, date, accepted)
  end

  def self.resource_counts_for_node_ids_and_date(node_ids, date, accepted)
    NodeBaseline.self_comparisons.count(
      :joins => :differences,
      :conditions => { :baseline_differences => { :accepted => accepted }, :node_id => node_ids, :date => date })
  end

  def self.node_count_for_group_and_date(group, date)
    NodeBaseline.self_comparisons.
      count(:conditions => { :node_id => group.nodes, :date => date })
  end
end
